diff --git a/configure.ac b/configure.ac
index e574430..b60ae23 100644
--- a/configure.ac
+++ b/configure.ac
@@ -2639,6 +2639,55 @@ then
 fi
 
 dnl
+dnl  mpg123 plugin
+dnl
+AC_ARG_ENABLE(mpg123,
+  [  --enable-mpg123            libmpg123 module (default disabled)])
+if test "${enable_mpg123}" = "yes"
+then
+  AC_ARG_WITH(mpg123,
+    [  --with-mpg123=PATH         path to libmpg123],[],[])
+  if test "${with_mpg123}" != "no" -a -n "${with_mpg123}"
+  then
+    VLC_ADD_CPPFLAGS([mpgatofixed32_mpg123],[-I${with_mpg123}/include])
+    VLC_ADD_LIBS([mpgatofixed32_mpg123],[-L${with_mpg123}/lib])
+  fi
+
+  AC_ARG_WITH(mpg123-tree,
+    [  --with-mpg123-tree=PATH    mpg123 tree for static linking],[],[])
+  if test "${with_mpg123_tree}" != "no" -a -n "${with_mpg123_tree}"
+  then
+    real_mpg123_tree="`cd ${with_mpg123_tree} 2>/dev/null && pwd`"
+    if test -z "${real_mpg123_tree}"
+    then
+      dnl  The given directory can't be found
+      AC_MSG_RESULT(no)
+      AC_MSG_ERROR([${with_mpg123_tree} directory doesn't exist])
+    fi
+    dnl  Use a custom libmpg123
+    AC_MSG_CHECKING(for mpg123.h in ${real_mpg123_tree})
+    if test -f ${real_mpg123_tree}/mpg123.h
+    then
+      AC_MSG_RESULT(yes)
+      VLC_ADD_CPPFLAGS([mpgatofixed32_mpg123],[-I${real_mpg123_tree}])
+      VLC_ADD_LIBS([mpgatofixed32_mpg123],[-L${real_mpg123_tree}/.libs -lmpg123])
+      VLC_ADD_PLUGIN([mpgatofixed32_mpg123])
+    else
+      AC_MSG_RESULT(no)
+      AC_MSG_ERROR([the specified tree doesn't have mpg123.h])
+    fi
+  else
+    CPPFLAGS="${CPPFLAGS_save} ${CPPFLAGS_mpgatofixed32_mpg123}"
+    AC_CHECK_HEADERS(mpg123.h, ,
+      [ AC_MSG_ERROR([Could not find libmpg123 on your system: you may get it from http://mpg123.com/. ]) ])
+    VLC_ADD_LIBS([mpgatofixed32_mpg123],[-lmpg123])
+    VLC_ADD_PLUGIN([mpgatofixed32_mpg123])
+    CPPFLAGS="${CPPFLAGS_save}"
+  fi
+fi
+
+
+dnl
 dnl   libid3tag support (FIXME!!! doesn't work with new input)
 dnl
 AC_ARG_ENABLE( id3tag,
diff --git a/modules/audio_filter/converter/Modules.am b/modules/audio_filter/converter/Modules.am
index 6aef6a3..30f572d 100644
--- a/modules/audio_filter/converter/Modules.am
+++ b/modules/audio_filter/converter/Modules.am
@@ -4,6 +4,7 @@ SOURCES_a52tofloat32 = a52tofloat32.c
 SOURCES_dtstospdif = dtstospdif.c
 SOURCES_dtstofloat32 = dtstofloat32.c
 SOURCES_mpgatofixed32 = mpgatofixed32.c
+SOURCES_mpgatofixed32_mpg123 = mpgatofixed32_mpg123.c
 SOURCES_audio_format = format.c
 
 libvlc_LTLIBRARIES += \
diff --git a/modules/audio_filter/converter/mpgatofixed32_mpg123.c b/modules/audio_filter/converter/mpgatofixed32_mpg123.c
new file mode 100755
index 0000000..02fdae1
--- /dev/null
+++ b/modules/audio_filter/converter/mpgatofixed32_mpg123.c
@@ -0,0 +1,243 @@
+/*****************************************************************************
+ * mpgatofixed32_mpg123.c: MPEG-1 & 2 audio layer I, II, III + MPEG 2.5 decoder,
+ * using MPG123
+ *****************************************************************************
+ * Copyright (C) 2001-2005 the VideoLAN team
+ * $Id$
+ *
+ * Authors: Christophe Massiot <massiot@via.ecp.fr>
+ *          Jean-Paul Saman <jpsaman _at_ videolan _dot_ org>
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
+ *****************************************************************************/
+
+/*****************************************************************************
+ * Preamble
+ *****************************************************************************/
+
+#include <mpg123.h>
+
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+#include <assert.h>
+
+#include <vlc_common.h>
+#include <vlc_plugin.h>
+#include <vlc_aout.h>
+#include <vlc_block.h>
+#include <vlc_filter.h>
+#include <vlc_cpu.h>
+
+/*****************************************************************************
+ * Local prototypes
+ *****************************************************************************/
+static int  OpenFilter ( vlc_object_t * );
+static void CloseFilter( vlc_object_t * );
+static block_t *Convert( filter_t *, block_t * );
+
+/*****************************************************************************
+ * Local structures
+ *****************************************************************************/
+struct filter_sys_t
+{
+    mpg123_handle * handle;
+    off_t           i_offset;
+    unsigned char * p_buffer;
+    size_t          i_size;
+
+    int             i_reject_count;
+};
+
+/*****************************************************************************
+ * Module descriptor
+ *****************************************************************************/
+vlc_module_begin ()
+    set_category( CAT_INPUT )
+    set_subcategory( SUBCAT_INPUT_ACODEC )
+    set_description( N_("MPEG audio decoder using mpg123") )
+    set_capability( "audio filter", 123 )
+    set_callbacks( OpenFilter, CloseFilter )
+vlc_module_end ()
+
+/*****************************************************************************
+ * DoWork: decode an MPEG audio frame.
+ *****************************************************************************/
+static void DoWork( filter_t * p_filter,
+                    aout_buffer_t * p_in_buf, aout_buffer_t * p_out_buf )
+{
+    filter_sys_t *p_sys = p_filter->p_sys;
+
+    p_out_buf->i_nb_samples = p_in_buf->i_nb_samples;
+    p_out_buf->i_buffer = p_in_buf->i_nb_samples * sizeof(vlc_fixed_t) *
+                               aout_FormatNbChannels( &p_filter->fmt_out.audio );
+
+    /* Do the actual decoding now. */
+    int i_err = mpg123_feed( p_sys->handle, p_in_buf->p_buffer,
+                                   p_in_buf->i_buffer );
+    if ( i_err != MPG123_OK || mpg123_decode_frame( p_sys->handle, &p_sys->i_offset, 
+         &p_sys->p_buffer, &p_sys->i_size ) != MPG123_OK )
+    {
+        msg_Dbg( p_filter, "libmpg123 error: %s",
+                 mpg123_strerror( p_sys->handle ) );
+        p_sys->i_reject_count = 3;
+    }
+    else if( p_in_buf->i_flags & BLOCK_FLAG_DISCONTINUITY )
+    { 
+        p_sys->i_reject_count = 3;
+    }
+
+    if( p_sys->i_reject_count > 0 )
+    {
+        if( p_filter->fmt_out.audio.i_format == VLC_CODEC_FL32 )
+        {
+            int i;
+            int i_size = p_out_buf->i_buffer / sizeof(float);
+
+            float * a = (float *)p_out_buf->p_buffer;
+            for ( i = 0 ; i < i_size ; i++ )
+                *a++ = 0.0;
+        }
+        else
+        {
+            memset( p_out_buf->p_buffer, 0, p_out_buf->i_buffer );
+        }
+        p_sys->i_reject_count--;
+        return;
+    }
+
+    /* Get number of channels. */
+    int i_channels;
+    mpg123_getformat( p_sys->handle, NULL, &i_channels, NULL );
+
+    /* libmpg123 outputs 16bit samples. */
+    unsigned int i_samples = p_sys->i_size / 2;
+
+    assert( i_samples / i_channels == p_out_buf->i_nb_samples );
+    if ( p_filter->fmt_out.audio.i_format == VLC_CODEC_FI32 )
+    {
+        /* Convert signed int16 to signed int32.*/
+        vlc_fixed_t * p_samples = (vlc_fixed_t *)p_out_buf->p_buffer;
+
+        while ( i_samples-- )
+        {
+            *p_samples++ = (*(int16_t *)p_sys->p_buffer) << 16;
+            p_sys->p_buffer += 2;
+        }
+    }
+    else
+    {
+        /* Convert signed int16 to float32. */
+        float * p_samples = (float *)p_out_buf->p_buffer;
+        const float f_temp = (float)0x8000;
+
+        while ( i_samples-- )
+        {
+            *p_samples++ = (float)*(int16_t *)p_sys->p_buffer / f_temp;
+            p_sys->p_buffer += 2;
+        }
+    }
+}
+
+/*****************************************************************************
+ * OpenFilter:
+ *****************************************************************************/
+static int OpenFilter( vlc_object_t *p_this )
+{
+    filter_t *p_filter = (filter_t *)p_this;
+    filter_sys_t *p_sys;
+
+    if( p_filter->fmt_in.i_codec != VLC_CODEC_MPGA &&
+        p_filter->fmt_in.i_codec != VLC_FOURCC('m','p','g','3') )
+        return VLC_EGENERIC;
+    if( !AOUT_FMTS_SIMILAR( &p_filter->fmt_in.audio, &p_filter->fmt_out.audio ) )
+        return VLC_EGENERIC;
+
+    /* Allocate the memory needed to store the module's structure */
+    p_sys = p_filter->p_sys = malloc( sizeof(filter_sys_t) );
+    if( p_sys == NULL )
+        return -1;
+    p_sys->i_reject_count = 0;
+
+    p_filter->pf_audio_filter = Convert;
+
+    /* Initialize libmpg123 */
+    if ( mpg123_init() != MPG123_OK )
+        return VLC_EGENERIC;
+    if ( (p_sys->handle = mpg123_new( NULL, NULL )) == NULL )
+        return VLC_EGENERIC;
+    if ( mpg123_open_feed( p_sys->handle ) != MPG123_OK )
+        return VLC_EGENERIC;
+    mpg123_param( p_sys->handle, MPG123_ADD_FLAGS, MPG123_NO_RESYNC, 0 );
+
+    p_filter->fmt_out.i_codec = HAVE_FPU ? VLC_CODEC_FL32 : VLC_CODEC_FI32;
+    p_filter->fmt_out.audio.i_format = p_filter->fmt_out.i_codec;
+    p_filter->fmt_out.audio.i_bitspersample =
+        aout_BitsPerSample( p_filter->fmt_out.i_codec );
+
+    msg_Dbg( p_this, "%4.4s->%4.4s, bits per sample: %i",
+             (char *)&p_filter->fmt_in.i_codec,
+             (char *)&p_filter->fmt_out.i_codec,
+             p_filter->fmt_in.audio.i_bitspersample );
+
+    return 0;
+}
+
+/*****************************************************************************
+ * CloseFilter : deallocate data structures
+ *****************************************************************************/
+static void CloseFilter( vlc_object_t *p_this )
+{
+    filter_t *p_filter = (filter_t *)p_this;
+    filter_sys_t *p_sys = p_filter->p_sys;
+
+    mpg123_close( p_sys->handle );
+    mpg123_delete( p_sys->handle );
+    mpg123_exit();
+    free( p_sys );
+}
+
+static block_t *Convert( filter_t *p_filter, block_t *p_block )
+{
+    if( !p_block || !p_block->i_nb_samples )
+    {
+        if( p_block )
+            block_Release( p_block );
+        return NULL;
+    }
+
+    size_t i_out_size = p_block->i_nb_samples *
+      p_filter->fmt_out.audio.i_bitspersample *
+        p_filter->fmt_out.audio.i_channels / 8;
+
+    block_t *p_out = p_filter->pf_audio_buffer_new( p_filter, i_out_size );
+    if( !p_out )
+    {
+        msg_Warn( p_filter, "can't get output buffer" );
+        block_Release( p_block );
+        return NULL;
+    }
+
+    p_out->i_nb_samples = p_block->i_nb_samples;
+    p_out->i_dts = p_block->i_dts;
+    p_out->i_pts = p_block->i_pts;
+    p_out->i_length = p_block->i_length;
+
+    DoWork( p_filter, p_block, p_out );
+
+    block_Release( p_block );
+
+    return p_out;
+}
diff --git a/modules/codec/mpeg_audio.c b/modules/codec/mpeg_audio.c
old mode 100644
new mode 100755
index 4267f09..9906eff
--- a/modules/codec/mpeg_audio.c
+++ b/modules/codec/mpeg_audio.c
@@ -465,7 +465,7 @@ static void *DecodeBlock( decoder_t *p_dec, block_t **pp_block )
                             p_buf, __MIN( (unsigned)p_sys->i_frame_size, p_out_buffer->i_buffer ) );
 
             /* Get beginning of next frame for libmad */
-            if( !p_sys->b_packetizer )
+            if( !p_sys->b_packetizer && !module_exists( "mpgatofixed32_mpg123" ) )
             {
                 assert( p_out_buffer->i_buffer >= (unsigned)p_sys->i_frame_size + MAD_BUFFER_GUARD );
                 memcpy( p_buf + p_sys->i_frame_size,
@@ -508,8 +508,12 @@ static uint8_t *GetOutBuffer( decoder_t *p_dec, block_t **pp_out_buffer )
     p_dec->fmt_out.audio.i_rate     = p_sys->i_rate;
     p_dec->fmt_out.audio.i_channels = p_sys->i_channels;
     p_dec->fmt_out.audio.i_frame_length = p_sys->i_frame_length;
-    p_dec->fmt_out.audio.i_bytes_per_frame =
-        p_sys->i_max_frame_size + MAD_BUFFER_GUARD;
+
+    if ( module_exists( "mpgatofixed32_mpg123" ) )
+        p_dec->fmt_out.audio.i_bytes_per_frame = p_sys->i_max_frame_size;
+    else
+        p_dec->fmt_out.audio.i_bytes_per_frame = 
+            p_sys->i_max_frame_size + MAD_BUFFER_GUARD;
 
     p_dec->fmt_out.audio.i_original_channels = p_sys->i_channels_conf;
     p_dec->fmt_out.audio.i_physical_channels =
@@ -551,8 +555,11 @@ static aout_buffer_t *GetAoutBuffer( decoder_t *p_dec )
         p_buf->i_flags |= BLOCK_FLAG_DISCONTINUITY;
     p_sys->b_discontinuity = false;
 
-    /* Hack for libmad filter */
-    p_buf = block_Realloc( p_buf, 0, p_sys->i_frame_size + MAD_BUFFER_GUARD );
+    /* Only apply hack for libmad filter if using libmad */
+    if ( module_exists( "mpgatofixed32_mpg123" ) )
+        p_buf = block_Realloc( p_buf, 0, p_sys->i_frame_size );
+    else
+        p_buf = block_Realloc( p_buf, 0, p_sys->i_frame_size + MAD_BUFFER_GUARD );
 
     return p_buf;
 }
